#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/errno.h>

#include <linux/pcmcia.h>
#include "pcmciad.h"

#ifdef DEBUG
#define syslog fprintf
#define LOG_NOTICE stderr
#define LOG_INFO stderr
#define LOG_ERR stderr
#endif

#define IOCTL(msg, cmd, arg) \
     do { int _val; _val=arg; IOCTL_PTR(msg,cmd,&_val); } while (0)
#define IOCTL_PTR(msg, cmd, arg) \
     do { if(ioctl(sock->fd,cmd,arg)<0) perror(msg); } while (0)


/*
 * Function: pcic_init_sock
 *
 * Purpose: Initialize a socket structure.
 *
 * Arguments:
 *
 * 	i	(r) pcmcia socket to examine
 * 	sock	(w) socket structure to be initialized
 *
 * Effects:
 * 
 * sock->fd is connected to PCMCIA socket (slot) i.  sock->valid and
 * sock->cardp are set to indicate whether the socket exists and
 * presently contains a card.
 */
void pcic_init_sock(int i, struct pcic_sock *sock)
{
     char buf[20];
     int id;

     sock->valid = sock->cardp = 0;

#ifndef USER_DRV
     sprintf(buf, "/dev/pc%d", i);
     sock->fd = open(buf, O_RDONLY);
     if (sock->fd < 0) {
	  sock->valid = 0;
	  if (errno != ENOENT)
	       syslog(LOG_ERR, "cannot open %s: %s\n", buf, strerror(errno));
	  return;
     }
#else
     sock->fd.i_rdev = i;
#endif
     
     if (ioctl(sock->fd, PCMCIA_GET_STATUS, &id) < 0) {
	  if (errno != ENODEV)
	       syslog(LOG_ERR, "cannot get status of socket %d: %s\n",
		      i, strerror(errno));
	  return;
     }

     sock->valid = 1;
     sock->cardp = (id & PC_DETECT) != 0;
}

/*
 * Function: pcic_parse_card
 *
 * Purpose: Parses the PCMCIA configuration tuples in a card.
 *
 * Arguments:
 *
 * 	sock	(r/w) the socket containing the card to be parsed
 *	print	(r) whether to print the tuples that are parsed
 * 
 * Requires: a card must be installed in the socket
 *
 * Effects:
 *
 * The PCMCIA configuration tuples for the card in socket sock are
 * parsed and stored in sock.  If print is true, the tuples are
 * printed to stdout.
 */
void pcic_parse_card(struct pcic_sock *sock, int print)
{
     int val;
     
     IOCTL_PTR("getting flags", PCMCIA_GET_FLAGS, &val);
     val |= PC_VCC_5V;
     IOCTL("powering up and resetting", PCMCIA_SET_FLAGS, val | PC_RESET);
     sleep(1);
     IOCTL("un-resetting card", PCMCIA_SET_FLAGS, val);
     
     IOCTL_PTR("getting attribute memory", PCMCIA_GET_ATTR_MEM,
	       &sock->attrmem);
     parse_tuples(sock, print > 1);
     if (print)
	  pcic_print_sock(sock);
}
